from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class ElectronicPrescription(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса %s' % type(self).__name__)
        self.__updateCases()

        servicestub.route('/connect/token', method='POST', callback=self.token)
        servicestub.route('/MedicationPrescription/$by-card', method='GET', callback=self.getMedicationPrescription)
        servicestub.route('/MedicationDispense/$register', method='POST', callback=self.registerMedicationDispense)
        servicestub.route('/MedicationDispense/$set-as-error', method='POST', callback=self.cancelMedicationDispense)

    def token(self):
        self.log.info("Запрос токена")
        methodPath = '/connect/token'

        return self.__getResult(methodPath)

    def getMedicationPrescription(self):
        self.log.info("Получение электронного рецепта по карте")

        params = dict(bottle.request.query.decode())
        patientIdentifier = params.get('patientIdentifier', '')
        self.log.info('patientIdentifier %s ' % patientIdentifier)

        methodPath = '/MedicationPrescription/' + patientIdentifier
        if methodPath not in self.cases:
            methodPath = '/MedicationPrescription/<patientIdentifier>'

        return self.__getResult(methodPath)

    def registerMedicationDispense(self):
        self.log.info('Отпуск лекарственного средства')
        methodPath = '/MedicationDispense/register'

        return self.__getResult(methodPath)

    def cancelMedicationDispense(self):
        self.log.info('Отмена отпуска лекарственного средства')
        methodPath = '/MedicationDispense/set-as-error'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            self.log.warn('Путь "%s" не существует' % methodPath)
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if not len(self.cases[methodPath]):
            self.log.warn('В кейсах не осталось значений')
            return None

        time.sleep(self.cases[methodPath][0].get('timeout', 0))
        body = json.dumps(self.cases[methodPath][0].get('response'))

        response = bottle.HTTPResponse(status=self.cases[methodPath][0].get('status'), body=body)
        response.add_header("Content-Type", "application/json")

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodPath][0]:
            return response

        self.cases[methodPath][0]['count'] -= 1
        if self.cases[methodPath][0]['count'] == 0:
            del self.cases[methodPath][0]

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'electronicprescription.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = ElectronicPrescription()
