from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class GpbSbp(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса GpbSbp')
        self.__updateCases()

        servicestub.route('/merchant/qrc-data', method='POST', callback=self.qrcData)
        servicestub.route('/merchant/qrc-status', method='POST', callback=self.qrcStatus)
        servicestub.route('/merchant/payment/v1/cash-activate-qrc/params', method='POST', callback=self.cashActivateQrc)
        servicestub.route('/merchant/transfer/return/prepare', method='POST', callback=self.refundPrepare)
        servicestub.route('/merchant/transfer/return/confirm', method='POST', callback=self.refundConfirm)
        servicestub.route('/operation/info', method='POST', callback=self.operationInfo)
        servicestub.route('/merchant/transfer/rollback', method='POST', callback=self.rollback)

    def qrcData(self):
        self.log.info("Регистрация платежной ссылки")
        self.__updateCases()
        return self.__getResult('/merchant/qrc-data')

    def qrcStatus(self, orderId=''):
        self.log.info("Запрос статуса платежа")
        return self.__getResult('/merchant/qrc-status')

    def cashActivateQrc(self):
        self.log.info("Активация кассовой ссылки")
        self.__updateCases()
        return self.__getResult('/merchant/payment/v1/cash-activate-qrc/params')

    def refundPrepare(self):
        self.log.info("Создание черновика возврата")
        return self.__getResult('/merchant/transfer/return/prepare')

    def refundConfirm(self):
        self.log.info("Подтверждение возврата")
        return self.__getResult('/merchant/transfer/return/confirm')

    def operationInfo(self):
        self.log.info("Статус возврата")
        return self.__getResult('/operation/info')

    def rollback(self):
        self.log.info("Отмена платежа")
        return self.__getResult('/merchant/transfer/rollback')

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath][0].get('timeout', 0))

        body = json.dumps(self.cases[methodPath][0].get('response'))
        statusCode = self.cases[methodPath][0]['status'] if 'status' in self.cases[methodPath][0] else 200
        response = bottle.HTTPResponse(status=statusCode, body=body)
        response.add_header("Content-Type", "application/json")

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodPath][0]:
            return response

        self.cases[methodPath][0]['count'] -= 1

        if self.cases[methodPath][0]['count'] == 0:
            del self.cases[methodPath][0]

        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            for file in ["gpbsbp"]:
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = GpbSbp()
